@@ -1,6 +1,6 @@
 package Dancer2::Core::Error;
 # ABSTRACT: Class representing fatal errors
-$Dancer2::Core::Error::VERSION = '0.166001';
+$Dancer2::Core::Error::VERSION = '0.300000';
 use Moo;
 use Carp;
 use Dancer2::Core::Types;
@@ -8,6 +8,9 @@ use Dancer2::Core::HTTP;
 use Data::Dumper;
 use Dancer2::FileUtils qw/path open_file/;
 use Sub::Quote;
+use Module::Runtime 'require_module';
+use Ref::Util qw< is_hashref >;
+use Clone qw(clone);
 
 has app => (
     is        => 'ro',
@@ -56,9 +59,7 @@ sub _build_title {
 }
 
 has template => (
-    is => 'ro',
-
-#    isa => sub { ref($_[0]) eq 'SCALAR' || ReadableFilePath->(@_) },
+    is      => 'ro',
     lazy    => 1,
     builder => '_build_error_template',
 );
@@ -100,7 +101,7 @@ sub _build_static_page {
 sub default_error_page {
     my $self = shift;
 
-    require Template::Tiny;
+    require_module('Template::Tiny');
 
     my $uri_base = $self->has_app && $self->app->has_request ?
         $self->app->request->uri_base : '';
@@ -162,15 +163,7 @@ sub full_message {
 
 has serializer => (
     is        => 'ro',
-    isa       => Sub::Quote::quote_sub(q{
-        use Safe::Isa;
-        # Perl5.8 not supported for $_DOES (yet) so use $_can + does
-        if ($_[0]) {
-            $_[0]->$_can('does')
-              && $_[0]->does('Dancer2::Core::Role::Serializer')
-              or die "does not have role Dancer2::Core::Role::Serializer";
-        }
-    }),
+    isa       => Maybe[ConsumerOf['Dancer2::Core::Role::Serializer']],
     builder   => '_build_serializer',
 );
 
@@ -210,7 +203,14 @@ has response => (
     default => sub {
         my $self = shift;
         my $serializer = $self->serializer;
+        # include server tokens in response ?
+        my $no_server_tokens = $self->has_app
+            ? $self->app->config->{'no_server_tokens'}
+            : defined $ENV{DANCER_NO_SERVER_TOKENS}
+                ? $ENV{DANCER_NO_SERVER_TOKENS}
+                : 0;
         return Dancer2::Core::Response->new(
+            server_tokens => !$no_server_tokens,
             ( serializer => $serializer )x!! $serializer
         );
     }
@@ -272,9 +272,11 @@ sub _build_content {
         return $content if defined $content;
     }
 
-    # It doesn't make sense to return a static page if show_errors is on
-    if ( !$self->show_errors && (my $content = $self->static_page) ) {
-        return $content;
+    # It doesn't make sense to return a static page for a 500 if show_errors is on
+    if ( !($self->show_errors && $self->status eq '500') ) {
+         if ( my $content = $self->static_page ) {
+             return $content;
+         }
     }
 
     if ($self->has_app && $self->app->config->{error_template}) {
@@ -330,75 +332,48 @@ sub backtrace {
     }
     $message ||= 'Wooops, something went wrong';
 
-    $message = '<pre class="error">' . _html_encode($message) . '</pre>';
+    my $html = '<pre class="error">' . _html_encode($message) . "</pre>\n";
 
     # the default perl warning/error pattern
-    my ( $file, $line ) = ( $message =~ /at (\S+) line (\d+)/ );
-
+    my ($file, $line) = $message =~ /at (\S+) line (\d+)/;
     # the Devel::SimpleTrace pattern
-    ( $file, $line ) = ( $message =~ /at.*\((\S+):(\d+)\)/ )
-      unless $file and $line;
+    ($file, $line) = $message =~ /at.*\((\S+):(\d+)\)/ unless $file and $line;
 
     # no file/line found, cannot open a file for context
-    return $message unless ( $file and $line );
+    return $html unless $file and $line;
 
     # file and line are located, let's read the source Luke!
-    my $fh = eval { open_file( '<', $file ) } or return $message;
+    my $fh = eval { open_file('<', $file) } or return $html;
     my @lines = <$fh>;
     close $fh;
 
-    my $backtrace = $message;
+    $html .= qq|<div class="title">$file around line $line</div>|;
 
-    $backtrace
-      .= qq|<div class="title">| . "$file around line $line" . "</div>";
+    # get 5 lines of context
+    my $start = $line - 5 > 1 ? $line - 5 : 1;
+    my $stop = $line + 5 < @lines ? $line + 5 : @lines;
 
-    $backtrace .= qq|<pre class="content">|;
+    $html .= qq|<pre class="content"><table class="context">\n|;
+    for my $l ($start .. $stop) {
+        chomp $lines[$l - 1];
 
-    $line--;
-    my $start = ( ( $line - 3 ) >= 0 ) ? ( $line - 3 ) : 0;
-    my $stop =
-      ( ( $line + 3 ) < scalar(@lines) ) ? ( $line + 3 ) : scalar(@lines);
-
-    for ( my $l = $start; $l <= $stop; $l++ ) {
-        chomp $lines[$l];
-
-        if ( $l == $line ) {
-            $backtrace
-              .= qq|<span class="nu">|
-              . tabulate( $l + 1, $stop + 1 )
-              . qq|</span> <span style="color: red;">|
-              . _html_encode( $lines[$l] )
-              . "</span>\n";
-        }
-        else {
-            $backtrace
-              .= qq|<span class="nu">|
-              . tabulate( $l + 1, $stop + 1 )
-              . "</span> "
-              . _html_encode( $lines[$l] ) . "\n";
-        }
+        $html .= $l == $line ? '<tr class="errline">' : '<tr>';
+        $html .= "<th>$l</th><td>" . _html_encode($lines[$l - 1]) . "</td></tr>\n";
     }
-    $backtrace .= "</pre>";
+    $html .= "</table></pre>\n";
 
-    return $backtrace;
-}
-
-sub tabulate {
-    my ( $number, $max ) = @_;
-    my $len = length($max);
-    return $number if length($number) == $len;
-    return " $number";
+    return $html;
 }
 
 sub dumper {
     my $obj = shift;
 
     # Take a copy of the data, so we can mask sensitive-looking stuff:
-    my %data     = %$obj;
-    my $censored = _censor( \%data );
+    my $data     = clone($obj);
+    my $censored = _censor( $data );
 
     #use Data::Dumper;
-    my $dd = Data::Dumper->new( [ \%data ] );
+    my $dd = Data::Dumper->new( [ $data ] );
     my $hash_separator = '  @@!%,+$$#._(--  '; # Very unlikely string to exist already
     my $prefix_padding = '  #+#+@%.,$_-!((  '; # Very unlikely string to exist already
     $dd->Terse(1)->Quotekeys(0)->Indent(1)->Sortkeys(1)->Pair($hash_separator)->Pad($prefix_padding);
@@ -452,17 +427,22 @@ sub get_caller {
 
 sub _censor {
     my $hash = shift;
-    if ( !$hash || ref $hash ne 'HASH' ) {
+    my $visited = shift || {};
+
+    unless ( $hash && is_hashref($hash) ) {
         carp "_censor given incorrect input: $hash";
         return;
     }
 
     my $censored = 0;
     for my $key ( keys %$hash ) {
-        if ( ref $hash->{$key} eq 'HASH' ) {
-            # Take a copy of the data, so we can hide sensitive-looking stuff:
-            $hash->{$key} = { %{ $hash->{$key} } };
-            $censored += _censor( $hash->{$key} );
+        if ( is_hashref( $hash->{$key} ) ) {
+            if (!$visited->{ $hash->{$key} }) {
+                # mark the new ref as visited
+                $visited->{ $hash->{$key} } = 1;
+
+                $censored += _censor( $hash->{$key}, $visited );
+            }
         }
         elsif ( $key =~ /(pass|card?num|pan|secret)/i ) {
             $hash->{$key} = "Hidden (looks potentially sensitive)";
@@ -502,7 +482,7 @@ Dancer2::Core::Error - Class representing fatal errors
 
 =head1 VERSION
 
-version 0.166001
+version 0.300000
 
 =head1 SYNOPSIS
 
@@ -566,17 +546,13 @@ attribute's response.
 
 =head2 backtrace
 
-Create a backtrace of the code where the error is caused.
+Show the surrounding lines of context at the line where the error was thrown.
 
 This method tries to find out where the error appeared according to the actual
 error message (using the C<message> attribute) and tries to parse it (supporting
 the regular/default Perl warning or error pattern and the L<Devel::SimpleTrace>
 output) and then returns an error-highlighted C<message>.
 
-=head2 tabulate
-
-Small subroutine to help output nicer.
-
 =head2 environment
 
 A main function to render environment information: the caller (using
@@ -612,7 +588,7 @@ Dancer Core Developers
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2015 by Alexis Sukrieh.
+This software is copyright (c) 2019 by Alexis Sukrieh.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
