@@ -1,12 +1,13 @@
 # ABSTRACT: Config role for Dancer2 core objects
 package Dancer2::Core::Role::ConfigReader;
-$Dancer2::Core::Role::ConfigReader::VERSION = '0.166001';
+$Dancer2::Core::Role::ConfigReader::VERSION = '0.300000';
 use Moo::Role;
 
 use File::Spec;
 use Config::Any;
 use Hash::Merge::Simple;
 use Carp 'croak';
+use Module::Runtime 'require_module';
 
 use Dancer2::Core::Factory;
 use Dancer2::Core;
@@ -111,21 +112,37 @@ sub _build_config_files {
     return [] unless defined $location;
 
     my $running_env = $self->environment;
-    my @exts = Config::Any->extensions;
+    my @available_exts = Config::Any->extensions;
     my @files;
 
-    foreach my $ext (@exts) {
-        foreach my $file ( [ $location, "config.$ext" ],
-            [ $self->environments_location, "$running_env.$ext" ] )
-        {
-            my $path = path( @{$file} );
-            next if !-r $path;
-
-            push @files, $path;
+    my @exts = @available_exts;
+    if (my $ext = $ENV{DANCER_CONFIG_EXT}) {
+        if (grep { $ext eq $_ } @available_exts) {
+            @exts = $ext;
+            warn "Only looking for configs ending in '$ext'\n" 
+                if $ENV{DANCER_CONFIG_VERBOSE};
+        } else {
+            warn "DANCER_CONFIG_EXT environment variable set to '$ext' which\n" .
+                 "is not recognized by Config::Any. Looking for config file\n" .
+                 "using default list of extensions:\n" .
+                 "\t@available_exts\n";
         }
     }
 
-    return [ sort @files ];
+    foreach my $file ( [ $location, "config" ],
+        [ $self->environments_location, $running_env ] )
+    {
+        foreach my $ext (@exts) {
+            my $path = path( $file->[0], $file->[1] . ".$ext" );
+            next if !-r $path;
+
+            # Look for *_local.ext files
+            my $local = path( $file->[0], $file->[1] . "_local.$ext" );
+            push @files, $path, ( -r $local ? $local : () );
+        }
+    }
+
+    return \@files;
 }
 
 sub _build_config {
@@ -136,7 +153,10 @@ sub _build_config {
 
     my $config = Hash::Merge::Simple->merge(
         $default,
-        map +( $self->load_config_file($_) ), @{ $self->config_files }
+        map {
+            warn "Merging config file $_\n" if $ENV{DANCER_CONFIG_VERBOSE};
+            $self->load_config_file($_) 
+        } @{ $self->config_files }
     );
 
     $config = $self->_normalize_config($config);
@@ -222,7 +242,7 @@ my $_normalizers = {
         my ($charset) = @_;
         return $charset if !length( $charset || '' );
 
-        require Encode;
+        require_module('Encode');
         my $encoding = Encode::find_encoding($charset);
         croak
           "Charset defined in configuration is wrong : couldn't identify '$charset'"
@@ -269,7 +289,7 @@ Dancer2::Core::Role::ConfigReader - Config role for Dancer2 core objects
 
 =head1 VERSION
 
-version 0.166001
+version 0.300000
 
 =head1 DESCRIPTION
 
@@ -305,8 +325,6 @@ Returns the name of the environment.
 
 List of all the configuration files.
 
-=head1 ATTRIBUTES
-
 =head1 METHODS
 
 =head2 settings
@@ -331,7 +349,7 @@ Dancer Core Developers
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2015 by Alexis Sukrieh.
+This software is copyright (c) 2019 by Alexis Sukrieh.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
